@@ -23,8 +23,9 @@
  */
 
 #ifndef _XOPEN_SOURCE /* POSIX_MADV_RANDOM */
-#define _XOPEN_SOURCE 600
+#define _XOPEN_SOURCE 700
 #endif
+/* (_ATFILE_SOURCE or _XOPEN_SOURCE >= 700) */
 #ifndef _ATFILE_SOURCE /* fstatat(), openat() */
 #define _ATFILE_SOURCE
 #endif
@@ -32,23 +33,9 @@
 #define _GNU_SOURCE 1
 #endif
 /* large file support needed for stat(),fstat() input file > 2 GB */
-#if defined(_AIX)
-#ifndef _LARGE_FILES
-#define _LARGE_FILES
-#endif
-#else /*#elif defined(__linux__) || defined(__sun) || defined(__hpux)*/
-#ifndef _FILE_OFFSET_BITS
-#define _FILE_OFFSET_BITS 64
-#endif
-#ifndef _LARGEFILE_SOURCE
-#define _LARGEFILE_SOURCE 1
-#endif
-#ifndef _LARGEFILE64_SOURCE
-#define _LARGEFILE64_SOURCE 1
-#endif
-#endif
+#define PLASMA_FEATURE_ENABLE_LARGEFILE
 
-/* Note: mcdb client does not use largefile defines when compiled 32-bit
+/* Note: mcdb client does not support files > 4 GB when compiled 32-bit
  * since the implementation mmap()s the entire file into the address space,
  * meaning only up to 4 GB (minus memory used by program and other libraries)
  * is supported in 32-bit.  djb's original implementation using lseek() and
@@ -59,9 +46,10 @@
 #include "mcdb.h"
 #include "nointr.h"
 #include "uint32.h"
-#include "code_attributes.h"
+#include "plasma/plasma_attr.h"
+#include "plasma/plasma_membar.h"
+#include "plasma/plasma_stdtypes.h"  /* SIZE_MAX */
 
-#include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
@@ -69,14 +57,13 @@
 #include <errno.h>
 #include <limits.h>
 #include <string.h>
-#include <stdint.h>    /* SIZE_MAX */
 
 #ifdef _THREAD_SAFE
-#include <pthread.h>       /* pthread_mutex_t, pthread_mutex_{lock,unlock}() */
-static pthread_mutex_t mcdb_global_mutex = PTHREAD_MUTEX_INITIALIZER;
+#include "plasma/plasma_spin.h" /* plasma_spin_lock_t, plasma_spin_lock_*() */
+static plasma_spin_lock_t mcdb_global_spinlock = PLASMA_SPIN_LOCK_INITIALIZER;
 #else
-#define pthread_mutex_lock(mutexp) 0
-#define pthread_mutex_unlock(mutexp) (void)0
+#define plasma_spin_lock_acquire(spin) (void)0
+#define plasma_spin_lock_release(spin) (void)0
 #endif
 
 #ifndef O_CLOEXEC /* O_CLOEXEC available since Linux 2.6.23 */
@@ -101,13 +88,25 @@ mcdb_findtagstart(struct mcdb * const restrict m,
                   const char * const restrict key, const size_t klen,
                   const unsigned char tagc)
 {
-    const uint32_t khash_init = /* init hash value; hash tagc if tagc not 0 */
-      (tagc != 0)
-        ? uint32_hash_djb_uchar(UINT32_HASH_DJB_INIT, tagc)
-        : UINT32_HASH_DJB_INIT;
-    const uint32_t khash = uint32_hash_djb(khash_init, key, klen);
     const unsigned char * restrict ptr;
+    uint32_t khash;
+    if (m->map->hash_fn == uint32_hash_djb) {
+        const uint32_t khash_init = /*init hash value; hash tagc if tagc not 0*/
+          (tagc != 0)
+            ? uint32_hash_djb_uchar(UINT32_HASH_DJB_INIT, tagc)
+            : UINT32_HASH_DJB_INIT;
+        khash = uint32_hash_djb(khash_init, key, klen);
+    }
+    else {
+        const uint32_t khash_init = /*init hash value; hash tagc if tagc not 0*/
+          (tagc != 0)
+            ? m->map->hash_fn(m->map->hash_init, (const char *)&tagc, 1u)
+            : m->map->hash_init;
+        khash = m->map->hash_fn(khash_init, key, klen);
+    }
 
+    /* (hash function should not change on refresh,
+     *  else move mcdb_thread_refresh_self() before khash calculation)*/
     (void) mcdb_thread_refresh_self(m);
     /* (ignore rc; continue with previous map in case of failure) */
 
@@ -121,9 +120,9 @@ mcdb_findtagstart(struct mcdb * const restrict m,
     /* (size of data in lvl2 hash table element is 16-bytes (shift 4 bits)) */
     m->kpos  = m->hpos
              +(((uintptr_t)((khash>>MCDB_SLOT_BITS) % m->hslots)) << m->map->b);
-    ptr = m->map->ptr + m->kpos;
-    __builtin_prefetch(ptr,0,2);    /*prefetch for mcdb_findtagnext()*/
-    __builtin_prefetch(ptr+64,0,2); /*prefetch for mcdb_findtagnext()*/
+    ptr = m->map->ptr + m->kpos;             /*prefetch for mcdb_findtagnext()*/
+    __builtin_prefetch(ptr,0,PLASMA_ATTR_MM_HINT_T1);
+    __builtin_prefetch(ptr+64,0,PLASMA_ATTR_MM_HINT_T1);
     uint32_strpack_bigendian_aligned_macro(&m->khash, khash);/*store bigendian*/
     return true;
 }
@@ -148,7 +147,7 @@ mcdb_findtagnext(struct mcdb * const restrict m,
             khash= *(uint32_t *)ptr; /* m->khash stored bigendian */
             vpos = uint32_strunpack_bigendian_aligned_macro(ptr+4);
             ptr  = mptr + vpos;
-            __builtin_prefetch((char *)ptr, 0, 1);
+            __builtin_prefetch((char *)ptr, 0, PLASMA_ATTR_MM_HINT_T2);
             if (__builtin_expect((!vpos), 0))
                 break;
             ++m->loop;
@@ -172,7 +171,7 @@ mcdb_findtagnext(struct mcdb * const restrict m,
             khash   = *(uint32_t *)ptr; /* m->khash stored bigendian */
             m->klen = uint32_strunpack_bigendian_aligned_macro(ptr+4);
             vpos    = uint64_strunpack_bigendian_aligned_macro(ptr+8);
-            __builtin_prefetch((char *)mptr+vpos+4, 0, 1);
+            __builtin_prefetch((char *)mptr+vpos+4, 0, PLASMA_ATTR_MM_HINT_T2);
             if (__builtin_expect((!vpos), 0))
                 break;
             ++m->loop;
@@ -253,7 +252,7 @@ mcdb_iter(struct mcdb_iter * const restrict iter)
             /* klen <= INT_MAX-8 (see mcdb_make.c), so no need to also check
              *   (iter->ptr >= iter->eod-(MCDB_PAD_MASK-7))
              * (using original iter->ptr value before update above) */
-            __builtin_prefetch(iter->ptr, 0, 3);
+            __builtin_prefetch(iter->ptr, 0, PLASMA_ATTR_MM_HINT_T0);
             return true;
         }
         iter->ptr = iter->eod;   /* (reached end of data; return false below) */
@@ -275,8 +274,8 @@ mcdb_iter_init(struct mcdb_iter * const restrict iter,
     unsigned char * const ptr = m->map->ptr;
     iter->ptr  = ptr + MCDB_HEADER_SZ;
     iter->eod  = ptr + uint64_strunpack_bigendian_aligned_macro(ptr) - 7;
-    __builtin_prefetch(iter->ptr,0,3); /*(must be non-faulting load if 0 recs)*/
-    iter->klen = 0;
+    __builtin_prefetch(iter->ptr,0,PLASMA_ATTR_MM_HINT_T0);
+    iter->klen = 0;                     /*(non-faulting prefetch ld if 0 recs)*/
     iter->dlen = 0;
     iter->map  = m->map;
     /* Note: callers that intend to iterate through entire mcdb might call
@@ -307,7 +306,8 @@ mcdb_mmap_unmap(struct mcdb_mmap * const restrict map)
     map->size = 0;    /* map->size initialization required for mcdb_read() */
 }
 
-bool  __attribute_noinline__
+__attribute_noinline__
+bool
 mcdb_mmap_init(struct mcdb_mmap * const restrict map, int fd)
 {
     struct stat st;
@@ -320,8 +320,8 @@ mcdb_mmap_init(struct mcdb_mmap * const restrict map, int fd)
     if (st.st_size > (off_t)SIZE_MAX) return (errno = EFBIG, false);
   #endif
     x = mmap(0, (size_t)st.st_size, PROT_READ, MAP_SHARED, fd, 0);
-    if (x == MAP_FAILED) return false;
-    __builtin_prefetch((char *)x+960, 0, 3); /*(touch mem page w/ mcdb header)*/
+    if (x == MAP_FAILED) return false;             /*(touch page w/ mcdb hdr)*/
+    __builtin_prefetch((char *)x, 0, PLASMA_ATTR_MM_HINT_T0);
   #if 0 /* disable; does not appear to improve performance */
     /*(peformance hit when hitting an uncached mcdb on my 32-bit Pentium-M)*/
     if (st.st_size > 4194304) /*(skip syscall overhead if < 4 MB (arbitrary))*/
@@ -335,10 +335,13 @@ mcdb_mmap_init(struct mcdb_mmap * const restrict map, int fd)
     map->mtime = st.st_mtime;
     map->next  = NULL;
     map->refcnt= 0;
+    map->hash_init = UINT32_HASH_DJB_INIT;
+    map->hash_fn   = uint32_hash_djb;
     return true;
 }
 
-void  __attribute_noinline__
+__attribute_noinline__
+void
 mcdb_mmap_prefault(const struct mcdb_mmap * const restrict map)
 {
     /* improves performance on uncached mcdb (if about to make *many* queries)
@@ -348,7 +351,8 @@ mcdb_mmap_prefault(const struct mcdb_mmap * const restrict map)
                   POSIX_MADV_WILLNEED | POSIX_MADV_RANDOM);
 }
 
-void  __attribute_noinline__
+__attribute_noinline__
+void
 mcdb_mmap_free(struct mcdb_mmap * const restrict map)
 {
     if (map == NULL) return;
@@ -358,22 +362,27 @@ mcdb_mmap_free(struct mcdb_mmap * const restrict map)
             map->fn_free(map->fname);
             map->fname = NULL;
         }
-        map->fn_free(map);
+        if (!map->allocated)
+            map->fn_free(map);
     }
 }
 
-void  __attribute_noinline__
+__attribute_noinline__
+void
 mcdb_mmap_destroy(struct mcdb_mmap * const restrict map)
 {
+  #ifdef AT_FDCWD
     if (map == NULL) return;
     if (map->dfd != -1) {
         (void) nointr_close(map->dfd);
         map->dfd = -1;
     }
+  #endif
     mcdb_mmap_free(map);
 }
 
-bool  __attribute_noinline__
+__attribute_noinline__
+bool
 mcdb_mmap_reopen(struct mcdb_mmap * const restrict map)
 {
     int fd;
@@ -401,6 +410,7 @@ bool
 mcdb_mmap_refresh_check(const struct mcdb_mmap * const restrict map)
 {
     struct stat st;
+    plasma_membar_ld_datadep(); /*(thread ld order dep b/w map and map->ptr)*/
     return (map->ptr == NULL
             || ( (
                   #ifdef AT_FDCWD
@@ -425,7 +435,7 @@ mcdb_mmap_refresh_check(const struct mcdb_mmap * const restrict map)
  *
  *   maintenance thread: mcdb_mmap_refresh_threadsafe(&map) (periodic/triggered)
  *   querying threads:   mcdb_thread_register(m)
- *   querying threads:   mcdb_find(m,key,klen)          (repeat for may lookups)
+ *   querying threads:   mcdb_find(m,key,klen)         (repeat for many lookups)
  *   querying threads:   mcdb_thread_unregister(m)
  *
  *   maintenance thread: mcdb_mmap_destroy(map)
@@ -443,7 +453,10 @@ mcdb_mmap_refresh_check(const struct mcdb_mmap * const restrict map)
  *
  * Note: use return value from mcdb_mmap_create().  If an error occurs during
  * mcdb_mmap_create(), fn_free(map) is called, whether or not map or NULL was
- * passed as first argument to mcdb_mmap_create().
+ * passed as first argument to mcdb_mmap_create().  If non-NULL map is passed
+ * in to mcdb_mmap_create(), then it is -not- free'd by mcdb_mmap_destroy(),
+ * though internal allocations and resources are free'd.  If NULL map is passed
+ * in, then it is free'd with mcdb_mmap_destroy() since mcdb allocated the map.
  */
 __attribute_noinline__
 struct mcdb_mmap *
@@ -454,6 +467,7 @@ mcdb_mmap_create(struct mcdb_mmap * restrict map,
 {
     char *fbuf;
     size_t flen;
+    const int allocated = (map != NULL);
 
     if (fn_malloc == NULL)
         return NULL;
@@ -463,10 +477,11 @@ mcdb_mmap_create(struct mcdb_mmap * restrict map,
     memset(map, '\0', sizeof(struct mcdb_mmap));
     map->fn_malloc = fn_malloc;
     map->fn_free   = fn_free;
+    map->allocated = allocated;
     map->dfd       = -1;
     flen           = strlen(fname);
 
-  #if defined(__linux__) || defined(__sun)
+  #if defined(AT_FDCWD)
     if (dname != NULL) {
         /* caller must have open STDIN, STDOUT, STDERR */
         map->dfd = nointr_open(dname, O_RDONLY|O_CLOEXEC, 0);
@@ -515,25 +530,31 @@ mcdb_mmap_create(struct mcdb_mmap * restrict map,
     }
 }
 
-bool  __attribute_noinline__
+__attribute_noinline__
+struct mcdb_mmap *
 mcdb_mmap_thread_registration(struct mcdb_mmap ** const restrict mapptr,
                               const int flags)
 {
     struct mcdb_mmap *map;
     struct mcdb_mmap *next = NULL;
+    struct mcdb_mmap *unmap = NULL;
     const bool register_use_incr = ((flags & MCDB_REGISTER_USE_INCR) != 0);
     #define register_use_decr (!register_use_incr)
 
-    if (!(flags & MCDB_REGISTER_MUTEX_UNLOCK_HOLD)
-        && pthread_mutex_lock(&mcdb_global_mutex) != 0)
-        return false;
+    /* use file-scoped global spinlock to protect scan of multiple mcdb_mmap's
+     * (care has been taken to avoid blocking operations while holding lock) */
 
-    /*map = *mapptr;*/  /*(ought to be preceded by StoreLoad memory barrier)*/
-    map = *(struct mcdb_mmap * volatile * restrict)mapptr;
-    if (map == NULL || (map->ptr == NULL && register_use_incr)) {
-        if (!(flags & MCDB_REGISTER_MUTEX_LOCK_HOLD))
-            pthread_mutex_unlock(&mcdb_global_mutex);
-        return !register_use_incr; /* succeed if unregister; fail if register */
+    if (__builtin_expect( (!(flags & MCDB_REGISTER_ALREADY_LOCKED)), 1))
+        (void) plasma_spin_lock_acquire(&mcdb_global_spinlock);
+    plasma_membar_ccfence();
+
+    map = *mapptr;
+
+    if (__builtin_expect( (map == NULL), 0)
+        || (__builtin_expect( (map->ptr == NULL), 0) && register_use_incr)) {
+        plasma_spin_lock_release(&mcdb_global_spinlock);
+        /* succeed if unregister; fail if register */ /*(NULL is failure)*/
+        return (struct mcdb_mmap *)(uintptr_t)(!register_use_incr);
         /* If registering, possibly detected race condition in which another
          * thread released final reference and mcdb was munmap()'d while current
          * thread waited for lock.  It is now invalid to attempt to register use
@@ -552,67 +573,81 @@ mcdb_mmap_thread_registration(struct mcdb_mmap ** const restrict mapptr,
     }
 
     if ((register_use_decr || next != NULL) && --map->refcnt == 0) {
-        /* (above handles refcnt decremented to zero and refcnt already zero) */
-        while ((next = map->next) != NULL && next->refcnt == 0) {
+        while ((next = map->next) != NULL && (next->refcnt & ~0x40000000) == 0){
             map->next = next->next;
-            next->fname = NULL;   /* do not free(next->fname) yet */
-            mcdb_mmap_free(next);
-        }
-        if (!(flags & MCDB_REGISTER_MUNMAP_SKIP)) {
-            map->fname = NULL;    /* do not free(map->fname) yet */
-            mcdb_mmap_free(map);
-            if (register_use_decr)
-                *mapptr = NULL;
+            next->next = unmap;
+            (unmap = next)->fname = NULL;   /* do not free(next->fname) yet */
         }
+        if (next != NULL)
+            next->refcnt &= ~0x40000000;    /* unmark; now oldest map in chain*/
+        map->next = unmap;
+        (unmap = map)->fname = NULL;        /* do not free(map->fname) yet */
+        if (register_use_decr)
+            *mapptr = NULL;
+        /*(map will be free'd but map value still not NULL for return val)*/
     }
     #undef register_use_decr
 
-    if (!(flags & MCDB_REGISTER_MUTEX_LOCK_HOLD))
-        pthread_mutex_unlock(&mcdb_global_mutex);
+    plasma_spin_lock_release(&mcdb_global_spinlock);
 
-    return true;
+    /* release unused maps after releasing lock to minimize time holding lock */
+    next = unmap;
+    while (next != NULL) {
+        next = (unmap = next)->next;
+        mcdb_mmap_free(unmap);
+    }
+
+    /* return map on which incr refcnt to avoid race after unlocking */
+    return map; /*(for decr refcnt, non-NULL is success, even if map free'd)*/
 }
 
 /* theaded programs (while multiple threads are using same struct mcdb_mmap)
- * must reopen and register (update refcnt on previous and new mcdb_mmap) while
- * holding a lock, or else there are race conditions with refcnt'ing. */
-bool  __attribute_noinline__
+ * must be registered with current *mapptr before calling this routine, or else
+ * there is a race condition where map might be removed out from under us. */
+__attribute_noinline__
+bool
 mcdb_mmap_reopen_threadsafe(struct mcdb_mmap ** const restrict mapptr)
 {
-    bool rc = true;
+    struct mcdb_mmap * const map = *mapptr;
+    struct mcdb_mmap *next;
+    bool rc;
 
-    if (pthread_mutex_lock(&mcdb_global_mutex) != 0)
+    /* use high bit of refcnt to guard that one thread attempts reopen
+     * (must lock since others modify refcnt while holding same spinlock) */
+    (void) plasma_spin_lock_acquire(&mcdb_global_spinlock);
+    if (map->next != NULL)
+        return NULL !=                      /* registration releases spinlock */
+          mcdb_mmap_thread_registration_h(mapptr, MCDB_REGISTER_USE_INCR
+                                                 |MCDB_REGISTER_ALREADY_LOCKED);
+    if ((rc = (map->refcnt < 0x80000000u))) /*(0!=(map->refcnt & 0x80000000u))*/
+        map->refcnt |= 0x80000000u;
+    plasma_spin_lock_release(&mcdb_global_spinlock);
+    if (!rc)
+        return true; /*other threads return, even though mcdb not reopened yet*/
+
+    if (__builtin_expect( (map->fn_malloc == NULL), 0)
+        || (next = map->fn_malloc(sizeof(struct mcdb_mmap))) == NULL)
+        return false; /*(misconfigured mcdb_mmap or map->fn_malloc failed)*/
+
+    memcpy(next, map, sizeof(struct mcdb_mmap));
+    next->ptr = NULL; /*(skip munmap() in mcdb_mmap_reopen())*/
+    if (map->fname == map->fnamebuf)
+        next->fname = next->fnamebuf;
+    rc = mcdb_mmap_reopen(next);
+    if (__builtin_expect((!rc), 0)) {
+        map->fn_free(next);
         return false;
-
-    if ((*mapptr)->next == NULL) {
-        const struct mcdb_mmap * const map = *mapptr;
-        struct mcdb_mmap *next;
-        if (map->fn_malloc != NULL  /*(else caller misconfigured mcdb_mmap)*/
-            && (next = map->fn_malloc(sizeof(struct mcdb_mmap))) != NULL) {
-            memcpy(next, map, sizeof(struct mcdb_mmap));
-            next->ptr = NULL;       /*(skip munmap() in mcdb_mmap_reopen())*/
-            if (map->fname == map->fnamebuf)
-                next->fname = next->fnamebuf;
-            if ((rc = mcdb_mmap_reopen(next)))
-                (*mapptr)->next = next;
-            else
-                map->fn_free(next);
-        }
-        else
-            rc = false; /* map->fn_malloc failed */
     }
-    /* else rc = true;  (map->next already updated e.g. while obtaining lock) */
-
-    if (rc) {
-        const int mcdb_flags_hold_lock =
-            MCDB_REGISTER_USE_INCR
-          | MCDB_REGISTER_MUTEX_UNLOCK_HOLD
-          | MCDB_REGISTER_MUTEX_LOCK_HOLD;
-        rc = mcdb_mmap_thread_registration_h(mapptr, mcdb_flags_hold_lock);
-    }
-
-    pthread_mutex_unlock(&mcdb_global_mutex);
-    return rc;
+    next->hash_init = map->hash_init;
+    next->hash_fn   = map->hash_fn;
+    next->refcnt   |= 0x40000000u;    /* flag to indicate not oldest in chain */
+    plasma_membar_StoreStore();
+    map->next       = next;
+    (void) plasma_spin_lock_acquire(&mcdb_global_spinlock);
+    map->refcnt    &= ~0x80000000u;         /* registration releases spinlock */
+    return NULL !=
+      mcdb_mmap_thread_registration_h(mapptr, MCDB_REGISTER_USE_INCR
+                                             |MCDB_REGISTER_ALREADY_LOCKED);
 }
 
 
@@ -621,32 +656,32 @@ mcdb_mmap_reopen_threadsafe(struct mcdb_mmap ** const restrict mapptr)
  * (optimization)
  * The aliases below are not a complete set of mcdb symbols,
  * but instead are the most common used in libnss_mcdb.so.2 */
-#if __GNUC_PREREQ(4,0) || __has_attribute(alias)
+#ifdef PLASMA_ATTR_ALIAS
 HIDDEN extern __typeof (mcdb_findtagstart)
                         mcdb_findtagstart_h
-  __attribute__((alias ("mcdb_findtagstart")));
+  __attribute_alias__ ("mcdb_findtagstart");
 HIDDEN extern __typeof (mcdb_findtagnext)
                         mcdb_findtagnext_h
-  __attribute__((alias ("mcdb_findtagnext")));
+  __attribute_alias__ ("mcdb_findtagnext");
 HIDDEN extern __typeof (mcdb_iter)
                         mcdb_iter_h
-  __attribute__((alias ("mcdb_iter")));
+  __attribute_alias__ ("mcdb_iter");
 HIDDEN extern __typeof (mcdb_iter_init)
                         mcdb_iter_init_h
-  __attribute__((alias ("mcdb_iter_init")));
+  __attribute_alias__ ("mcdb_iter_init");
 HIDDEN extern __typeof (mcdb_mmap_create)
                         mcdb_mmap_create_h
-  __attribute__((alias ("mcdb_mmap_create")));
+  __attribute_alias__ ("mcdb_mmap_create");
 HIDDEN extern __typeof (mcdb_mmap_destroy)
                         mcdb_mmap_destroy_h
-  __attribute__((alias ("mcdb_mmap_destroy")));
+  __attribute_alias__ ("mcdb_mmap_destroy");
 HIDDEN extern __typeof (mcdb_mmap_refresh_check)
                         mcdb_mmap_refresh_check_h
-  __attribute__((alias ("mcdb_mmap_refresh_check")));
+  __attribute_alias__ ("mcdb_mmap_refresh_check");
 HIDDEN extern __typeof (mcdb_mmap_thread_registration)
                         mcdb_mmap_thread_registration_h
-  __attribute__((alias ("mcdb_mmap_thread_registration")));
+  __attribute_alias__ ("mcdb_mmap_thread_registration");
 HIDDEN extern __typeof (mcdb_mmap_reopen_threadsafe)
                         mcdb_mmap_reopen_threadsafe_h
-  __attribute__((alias ("mcdb_mmap_reopen_threadsafe")));
+  __attribute_alias__ ("mcdb_mmap_reopen_threadsafe");
 #endif
