@@ -3,20 +3,20 @@
  *
  * Copyright (c) 2010, Glue Logic LLC. All rights reserved. code()gluelogic.com
  *
- *  This file is part of mcdb.
+ *  This file is part of plasma.
  *
- *  mcdb is free software: you can redistribute it and/or modify it under
+ *  plasma is free software: you can redistribute it and/or modify it under
  *  the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation, either version 2.1 of the License, or
  *  (at your option) any later version.
  *
- *  mcdb is distributed in the hope that it will be useful,
+ *  plasma is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU Lesser General Public License
- *  along with mcdb.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with plasma.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef INCLUDED_PLASMA_ATTR_H
@@ -327,6 +327,15 @@
 #endif
 #endif
 
+#ifndef __attribute_may_alias__
+#if __has_attribute(may_alias) \
+ || __GNUC_PREREQ(3,3)
+#define __attribute_may_alias__  __attribute__((__may_alias__))
+#else
+#define __attribute_may_alias__
+#endif
+#endif
+
 #ifndef __attribute_common__
 #if __has_attribute(common) \
  || __GNUC_PREREQ(3,3) \
@@ -478,6 +487,7 @@
 
 /*
  * Symbol visibility attributes:
+ * http://gcc.gnu.org/wiki/Visibility
  * http://docs.oracle.com/cd/E19205-01/820-4155/c.html
  * http://docs.sun.com/source/820-4155/c.html
  *   (Oracle/Sun Studio 12 C compiler supports __attribute__)
@@ -523,7 +533,7 @@
 #if __has_builtin(__builtin_constant_p) \
  || __GNUC_PREREQ(3,1)/*(check major,minor ver; actually supported in 3.0.1)*/\
  || defined(__IBMC__) || defined(__IBMCPP__) \
- || (defined(__SUNPRO_C)  && __SUNPRO_C  >= 0x5110) /* Sun Studio 12.2 C   */
+ || (defined(__SUNPRO_C)  && __SUNPRO_C  >= 0x5100) /* Sun Studio 12.1 C   */
 #define plasma_attr_has_builtin_constant_p
 #else
 #ifndef __builtin_constant_p
@@ -723,22 +733,26 @@ enum plasma_attr_mm_hint
  * (http://en.cppreference.com/w/cpp/language/alignas)
  *
  * portability: __attribute__((aligned)) can be at beginning or end of statement
- * except for xlC which supports __attribute__((aligned)) only at end of line.
- * MS __declspec() is expected early in statement, not at end.
+ * but must be after 'struct' (not before it) or else might be ignored.
+ * MS __declspec() is expected early in statement, not at end. 
  * Placement requirements makes use of this attribute less portable without
  * use of redundant statements: place __declspec_align__(x) at the beginning
  * of a statement and __attribute_aligned__(x) at the end of a statement.
  *
  * Oracle Solaris Studio C++ 12.2 finally adds support for aligned attribute.
  * Earlier versions support #pragma align, but mangled names must be used when
- * #pragma align is used inside a namespace.
+ * #pragma align is used inside a namespace, and local, stack vars not supported
  *   http://docs.oracle.com/cd/E18659_01/html/821-1383/bkbjx.html#bkbjy
  * Solaris standard libc does provide memalign() for allocating aligned memory,
- * but only #pragma align for stack or global alignment, until Sun Studio 12 C
- * and Sun Studio 12.2 C++ (according to doc, but syntax accepted in 12.1 C++).
+ * but only #pragma align for static or global alignment, until Sun Studio 12 C
+ * and Sun Studio 12.2 C++.
  * POSIX documents posix_memalign() which some platforms support.
  * C11 provides aligned_alloc()
  *
+ * Later patch releases of Sun Studio 12 update 1 C++ add attribute aligned.
+ *   CC: Sun C++ 5.10 SunOS_sparc Patch 128228-23  (output from $ CC -V )
+ *   https://getupdates.oracle.com/readme/README.128228-23
+ *
  * Misaligned memory access -- loads and stores -- can have different penalties
  * depending on whether the access is within a cache line, crosses a cache line,
  * or crosses a memory page boundary.  Misaligned memory access on SPARC has
@@ -749,11 +763,11 @@ enum plasma_attr_mm_hint
  */
 #ifndef __attribute_aligned__
 #if __has_attribute(aligned) \
- || defined(__GNUC__) /* __GNUC_PREREQ(?,?) */ \
+ || defined(__GNUC__) /* __GNUC_PREREQ(2,95) and possibly earlier */ \
  || defined(__xlc__) || defined(__xlC__) /* IBM AIX xlC */ \
  || (defined(__SUNPRO_C)  && __SUNPRO_C  >= 0x590)  /* Sun Studio 12   C   */ \
  || (defined(__SUNPRO_CC) && __SUNPRO_CC >= 0x5100) /* Sun Studio 12.1 C++ */ \
- || defined(__HP_cc) || defined(__HP_aCC)
+ || defined(__HP_cc) || defined(__HP_aCC)           /* (need Patch 128228-23)*/
 #define __attribute_aligned__(x)  __attribute__((__aligned__ (x)))
 #else
 #define __attribute_aligned__(x)
