@@ -6,20 +6,20 @@
  *
  * Copyright (c) 2012, Glue Logic LLC. All rights reserved. code()gluelogic.com
  *
- *  This file is part of mcdb.
+ *  This file is part of plasma.
  *
- *  mcdb is free software: you can redistribute it and/or modify it under
+ *  plasma is free software: you can redistribute it and/or modify it under
  *  the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation, either version 2.1 of the License, or
  *  (at your option) any later version.
  *
- *  mcdb is distributed in the hope that it will be useful,
+ *  plasma is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU Lesser General Public License
- *  along with mcdb.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with plasma.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef INCLUDED_PLASMA_SPIN_H
@@ -211,7 +211,7 @@ PLASMA_ATTR_Pragma_once
 #elif defined(PLASMA_FEATURE_POSIX)
   #include <unistd.h>  /* _POSIX_PRIORITY_SCHEDULING */
   #include <poll.h>    /* might also need _XOPEN_SOURCE=500 or better on _AIX */
-  #ifdef _POSIX_PRIORITY_SCHEDULING
+  #if defined(_POSIX_PRIORITY_SCHEDULING) && (_POSIX_PRIORITY_SCHEDULING-0)!= -1
   #include <sched.h>
   #define plasma_spin_yield() do {if (sched_yield()!=0) poll(NULL,0,1);}while(0)
   #else
@@ -466,6 +466,7 @@ plasma_spin_tktlock_release (plasma_spin_tktlock_t * const restrict spin)
      * ensure that pointer aliasing in union is not optimized into incorrect
      * code.  Might need to disable optimization on this routine or set
      * compiler optimization fences if atomic store does not provide barrier */
+    __attribute_may_alias__
   #if defined(__LITTLE_ENDIAN__)
     uint16_t * const ptr = &spin->lck.t.le;
   #elif defined(__BIG_ENDIAN__)
