@@ -3,20 +3,20 @@
  *
  * Copyright (c) 2013, Glue Logic LLC. All rights reserved. code()gluelogic.com
  *
- *  This file is part of mcdb.
+ *  This file is part of plasma.
  *
- *  mcdb is free software: you can redistribute it and/or modify it under
+ *  plasma is free software: you can redistribute it and/or modify it under
  *  the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation, either version 2.1 of the License, or
  *  (at your option) any later version.
  *
- *  mcdb is distributed in the hope that it will be useful,
+ *  plasma is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU Lesser General Public License
- *  along with mcdb.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with plasma.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -31,6 +31,9 @@
 
 /* note: tests int32_t and int64_t sizes; char and short not tested below */
 
+/* note: plasma_atomic.t.c fails to compile with (xlC 10.1) xlc_r -q32 -O2
+ * (xlc generates negative .line in asm (.line -4) (compile with -S to see)) */
+
 #include "../plasma_atomic.h"
 #include "../plasma_attr.h"
 #include "../plasma_membar.h"
@@ -38,6 +41,10 @@
 #include "../plasma_sysconf.h"
 #include "../plasma_test.h"
 
+#ifdef PLASMA_FEATURE_POSIX
+#include <unistd.h>  /* alarm() */
+#endif
+
 #if defined(__IBMC__) || defined(__IBMCPP__)
 #if !defined(_LP64) && !defined(__LP64__) && !defined(_ARCH_PPC64)
 #error "plasma_atomic.h 64-bit ops require: xlc -qarch=pwr5 or better 64-bit POWER arch"
@@ -735,7 +742,7 @@ plasma_atomic_t_relaxed_ptr (void)
      *      and store-conditional always fails (and we loop on lost reservation)
      */
     /* xor: 0 (identity) */
-    x32 = 0xFFFFFFFF;
+    x32 = (void *)(intptr_t)0xFFFFFFFF;
     r32 = plasma_atomic_fetch_xor_ptr(&x32, 0, memory_order_relaxed);
     plasma_membar_ccfence();
     rc &= PLASMA_TEST_COND(r32 == x32);
@@ -1117,6 +1124,12 @@ main (int argc, char *argv[])
     long nprocs = plasma_sysconf_nprocessors_onln();
     if (nprocs < 1)
         nprocs = 1;
+    if (nprocs > 32)
+        nprocs = 32;  /*(place upper bound on num threads created in tests)*/
+    /* expect all tests to complete in <2 mins  (revisit as necessary)
+     * (threaded tests take more time as CPU core count increases) */
+    alarm(120);
+
     rc &= plasma_atomic_t_relaxed();
     rc &= plasma_atomic_t_nthreads_add((int)nprocs);
     return !rc;
